#ifndef _CONFIG_H
#define _CONFIG_H

// {73AEE010-C0B9-469d-B86F-919A8DD6DD8F}
static const GUID guid_tab_pad_width = 
{ 0x73aee010, 0xc0b9, 0x469d, { 0xb8, 0x6f, 0x91, 0x9a, 0x8d, 0xd6, 0xdd, 0x8f } };
// {223A2511-4B0D-442f-A34D-946FD656D7C8}
static const GUID guid_tab_pad_height = 
{ 0x223a2511, 0x4b0d, 0x442f, { 0xa3, 0x4d, 0x94, 0x6f, 0xd6, 0x56, 0xd7, 0xc8 } };
// {43FEA661-976C-4bc1-B17B-DA3F8D29B19C}
static const GUID guid_min_height = 
{ 0x43fea661, 0x976c, 0x4bc1, { 0xb1, 0x7b, 0xda, 0x3f, 0x8d, 0x29, 0xb1, 0x9c } };
// {6F611C7F-B225-4adb-BBD7-1A2BDF630820}
static const GUID guid_font = 
{ 0x6f611c7f, 0xb225, 0x4adb, { 0xbb, 0xd7, 0x1a, 0x2b, 0xdf, 0x63, 0x8, 0x20 } };
// {FB92436C-282E-4030-930A-7D7B385F73C2}
static const GUID guid_config = 
{ 0xfb92436c, 0x282e, 0x4030, { 0x93, 0xa, 0x7d, 0x7b, 0x38, 0x5f, 0x73, 0xc2 } };

static cfg_int cfg_tab_pad_width(guid_tab_pad_width,6);
static cfg_int cfg_tab_pad_height(guid_tab_pad_height,3);
static cfg_int cfg_min_height(guid_min_height,0);

inline static LOGFONT get_def_font()
{
	LOGFONT foo;
	uGetMenuFont(&foo);
	return foo;
}
static cfg_struct_t<LOGFONT> cfg_font(guid_font,get_def_font());

static BOOL CALLBACK config_proc (HWND wnd, UINT msg, WPARAM wp, LPARAM lp);

class uieh_tabs_config : public preferences_page
{
public:
	virtual HWND create(HWND parent)
	{
		return uCreateDialog(IDD_CONFIG,parent,config_proc,0);
	}
	virtual const char * get_name() { return "Tabbed panel stack"; }

	virtual GUID get_guid(){
		return guid_config;
	};
	//! Retrieves GUID of parent page/branch of this page. See preferences_page::guid_* constants for list of standard parent GUIDs. Can also be a GUID of another page or a branch (see: preferences_branch).
	virtual GUID get_parent_guid(){
		return guid_tools;
	}
	//! Queries whether this page supports "reset page" feature.
	virtual bool reset_query(){
		return false;
	}
	//! Activates "reset page" feature. It is safe to assume that the preferences page dialog does not exist at the point this is called (caller destroys it before calling reset and creates it again afterwards).
	virtual void reset(){}
	//! Retrieves help URL. Without overriding it, it will redirect to foobar2000 wiki.

};

class cfg_var_notify_wnd
{
	list_t<HWND> g_wnd_list;
public:
	void on_var_change(UINT msg, WPARAM wp, cfg_var * ptr) 
	{
		int val = *(cfg_int *)ptr;

		unsigned n, count = g_wnd_list.get_count();
		for (n = 0; n < count; n++) 
		{
			uSendMessage(g_wnd_list[n], msg, wp, val);
		}
	}

public:
	cfg_var_notify_wnd(){};

	~cfg_var_notify_wnd(){
		g_wnd_list.remove_all();
	};

	void add_window(HWND wnd) 
	{
		g_wnd_list.add_item(wnd);
	}

	void remove_window(HWND wnd) 
	{
		g_wnd_list.remove_item(wnd);
	}
};

cfg_var_notify_wnd g_var_notify;
#define SEND_VAR_MSG(x, y) g_var_notify.on_var_change((x), 0, (&y))

enum { MIN_HEIGHT_CHANGED = WM_USER, TAB_PADDING_CHANGED, FONT_CHANGED, HIDETAB_CHANGED };



static BOOL CALLBACK config_proc (HWND wnd, UINT msg, WPARAM wp, LPARAM lp)
{
	static bool inited = false;

	switch(msg)
	{
	case WM_INITDIALOG:
		uSendDlgItemMessage(wnd,IDC_PADDING_HEIGHT_SPIN,UDM_SETRANGE,0,MAKELONG(99,0));
		uSendDlgItemMessage(wnd,IDC_PADDING_HEIGHT_SPIN,UDM_SETPOS,0,MAKELONG((int)cfg_tab_pad_height,0));
		uSendDlgItemMessage(wnd,IDC_PADDING_WIDTH_SPIN,UDM_SETRANGE,0,MAKELONG(99,0));
		uSendDlgItemMessage(wnd,IDC_PADDING_WIDTH_SPIN,UDM_SETPOS,0,MAKELONG((int)cfg_tab_pad_width,0));
		uSendDlgItemMessage(wnd,IDC_MIN_HEIGHT_SPIN,UDM_SETRANGE,0,MAKELONG(999,0));
		uSendDlgItemMessage(wnd,IDC_MIN_HEIGHT_SPIN,UDM_SETPOS,0,MAKELONG((int)cfg_min_height,0));
		inited = true;
		return 1;
	case WM_DESTROY:
		inited = false;
		break;
	case WM_COMMAND:
		switch(wp)
		{
		case IDC_PADDING_HEIGHT | (EN_CHANGE<<16):
			{
				if (!inited) break;
				BOOL result;
				unsigned x = GetDlgItemInt(wnd, IDC_PADDING_HEIGHT, &result, FALSE);
				if (result){
					cfg_tab_pad_height = x<100? x : 99;
					SEND_VAR_MSG(TAB_PADDING_CHANGED, cfg_tab_pad_height);
				}
			}
			break;
		case IDC_PADDING_WIDTH | (EN_CHANGE<<16):
			{
				if (!inited) break;
				BOOL result;
				unsigned x = GetDlgItemInt(wnd, IDC_PADDING_WIDTH, &result, FALSE);
				if (result){
					cfg_tab_pad_width = x<100? x : 99;
					SEND_VAR_MSG(TAB_PADDING_CHANGED, cfg_tab_pad_width);
				}
			}
			break;
		case IDC_MIN_HEIGHT | (EN_CHANGE<<16):
			{
				if (!inited) break;
				BOOL result;
				unsigned x = GetDlgItemInt(wnd, IDC_MIN_HEIGHT, &result, FALSE);
				if (result){
					cfg_min_height = x<1000? x : 999;
					SEND_VAR_MSG(MIN_HEIGHT_CHANGED, cfg_min_height);
				}
			}
			break;
		case IDC_SETFONT:
			{
				LOGFONT tmp = cfg_font;
				// From ye olde pfc
				CHOOSEFONT cf;
				memset(&cf,0,sizeof(cf));
				cf.lStructSize = sizeof(cf);
				cf.hwndOwner = wnd;
				cf.lpLogFont = &tmp;
				cf.Flags = CF_SCREENFONTS|CF_FORCEFONTEXIST|CF_INITTOLOGFONTSTRUCT;
				cf.nFontType = SCREEN_FONTTYPE;		
				if (ChooseFont(&cf)){
					cfg_font = tmp;
					SEND_VAR_MSG(FONT_CHANGED, cfg_font);
				}
			}
			break;
		}
		break;
	}
	return 0;
}


/*
static cfg_var_callback_toggle g_notify_min_height(&cfg_min_height, MIN_HEIGHT_CHANGED);
static cfg_var_callback_toggle g_notify_tab_padding_x(&cfg_tab_pad_width, TAB_PADDING_CHANGED);
static cfg_var_callback_toggle g_notify_tab_padding_y(&cfg_tab_pad_height, TAB_PADDING_CHANGED);
static cfg_var_callback_toggle g_notify_font(&cfg_font, FONT_CHANGED);
*/

#define REGISTER_PADDING_NOTIFY(x) g_var_notify.add_window(x)
#define UNREGISTER_PADDING_NOTIFY(x) g_var_notify.remove_window(x)

static preferences_page_factory_t<uieh_tabs_config> g_uieh_tabs_config;

#endif