#include "../SDK/foobar2000.h"
#include "foo_uie_tabs.h"

class mainmenu_groups_tabs
{
public:
    static const GUID tabs, hide_tab, change_height, select_tab;
};

class mainmenu_commands_select_tab : public mainmenu_commands
{
private:
    int get_tab_position(t_uint32 p_index, int & p_panel_num, int & p_menu_position) {
        pfc::ptr_list_t<uieh_tabs> list = uieh_tabs::g_get_list_tabs();
        for (int i = 0; i != list.get_count(); ++i) {
            if (p_index == 0 || p_index == 1) {
                p_panel_num = i;
                p_menu_position = p_index;
                return -1;
            }
            p_index -= 2;
            if (list.get_item_ref(i)->get_exts().get_count() > p_index) {
                p_panel_num = i;
                p_menu_position = p_index + 2;
                return p_index;
            }
            p_index -= static_cast<t_uint32>(list.get_item_ref(i)->get_exts().get_count());
        }
        p_panel_num = p_menu_position = -1;
        return -1;
    };
    pfc::string8 get_caption(t_uint32 p_index) {
        pfc::ptr_list_t<uieh_tabs> list = uieh_tabs::g_get_list_tabs();
        int tab_position, panel_num, menu_position;
        tab_position = get_tab_position(p_index, panel_num, menu_position);
        if (panel_num != -1) {
            // Cycle Forward
            if (menu_position == 0) {
                return pfc::string8() << "Panel #" << panel_num << " Cycle Forward";
            }
            // Cycle Backward
            if (menu_position == 1) {
                return pfc::string8() << "Panel #" << panel_num << " Cycle Backward";
            }
            return list.get_item_ref(panel_num)->get_exts().get_item_ref(tab_position)->caption;
        }
        return "";
    }

public:
    //! Retrieves number of implemented commands. Index parameter of other methods must be in 0....command_count-1 range.
    virtual t_uint32 get_command_count() {
        t_size num = 0;
        pfc::ptr_list_t<uieh_tabs> list = uieh_tabs::g_get_list_tabs();
        for (int i = 0; i != list.get_count(); ++i) {
            num += list.get_item_ref(i)->get_exts().get_count() + 2;
        }
        return static_cast<t_uint32>(num);
    };
	//! Retrieves GUID of specified command.
    virtual GUID get_command(t_uint32 p_index) {
        static_api_ptr_t<hasher_md5> md5;
        return md5->process_single_guid(get_caption(p_index), pfc::strlen_utf8(get_caption(p_index)));
    };
	//! Retrieves name of item, for list of commands to assign keyboard shortcuts to etc.
    virtual void get_name(t_uint32 p_index,pfc::string_base & p_out) {
        p_out = get_caption(p_index);
    };
	//! Retrieves item's description for statusbar etc.
    virtual bool get_description(t_uint32 p_index,pfc::string_base & p_out)
    {
        int tab_position, panel_num, menu_position;
        tab_position = get_tab_position(p_index, panel_num, menu_position);
        if (menu_position == 0) {
            p_out = pfc::string8("Circulates tabs on Panel #") << panel_num << " forward.";
        } else if (menu_position == 1) {
            p_out = pfc::string8("Circulates tabs on Panel #") << panel_num << " backword.";
        } else {
            p_out = pfc::string8("Selects \"") << get_caption(p_index) << "\" tab on Panel #" << panel_num << ".";
        }
        return true;
    };
	//! Retrieves GUID of owning menu group.
    virtual GUID get_parent() { return mainmenu_groups_tabs::select_tab; };
	//! Retrieves sorting priority of the command; the lower the number, the upper in the menu your commands will appear. Third party components should use sorting_priority_base and up (values below are reserved for internal use). In case of equal priority, order is undefined.
	virtual t_uint32 get_sort_priority() {return sort_priority_dontcare;}
	//! Retrieves display string and display flags to use when menu is about to be displayed. If returns false, menu item won't be displayed. You can create keyboard-shortcut-only commands by always returning false from get_display().
	virtual bool get_display(t_uint32 p_index,pfc::string_base & p_text,t_uint32 & p_flags)
    {
        pfc::ptr_list_t<uieh_tabs> list = uieh_tabs::g_get_list_tabs();
        int tab_position, panel_num, menu_position;
        tab_position = get_tab_position(p_index, panel_num, menu_position);
        if (TabCtrl_GetCurSel(list.get_item_ref(panel_num)->get_tab_wnd()) == tab_position) {
            p_flags = flag_checked;
        }
        get_name(p_index,p_text);
        return true;
    };
	//! Executes the command. p_callback parameter is reserved for future use and should be ignored / set to null pointer.
    virtual void execute(t_uint32 p_index,service_ptr_t<service_base> p_callback) {
        pfc::ptr_list_t<uieh_tabs> list = uieh_tabs::g_get_list_tabs();
        int tab_position, panel_num, menu_position;
        tab_position = get_tab_position(p_index, panel_num, menu_position);
        if (panel_num != -1) {
            // Cycle Forward
            if (menu_position == 0) {
                int tab_index = TabCtrl_GetCurSel(list.get_item_ref(panel_num)->get_tab_wnd()) + 1;
                if (tab_index >= TabCtrl_GetItemCount(list.get_item_ref(panel_num)->get_tab_wnd())) {
                    tab_index = 0;
                }
                TabCtrl_SetCurSel(list.get_item_ref(panel_num)->get_tab_wnd(), tab_index);
                list.get_item_ref(panel_num)->on_tab_sel_change(tab_index);
                return;
            }
            // Cycle Forward
            if (menu_position == 1) {
                int tab_index = TabCtrl_GetCurSel(list.get_item_ref(panel_num)->get_tab_wnd()) - 1;
                if (tab_index < 0) {
                    tab_index = TabCtrl_GetItemCount(list.get_item_ref(panel_num)->get_tab_wnd()) - 1;
                }
                TabCtrl_SetCurSel(list.get_item_ref(panel_num)->get_tab_wnd(), tab_index);
                list.get_item_ref(panel_num)->on_tab_sel_change(tab_index);
                return;
            }
            // Select Tab
            TabCtrl_SetCurSel(list.get_item_ref(panel_num)->get_tab_wnd(), tab_position);
            list.get_item_ref(panel_num)->on_tab_sel_change(tab_position);
            return;
        }
    };
};

class mainmenu_commands_hide_tab : public mainmenu_commands
{
public:
    //! Retrieves number of implemented commands. Index parameter of other methods must be in 0....command_count-1 range.
    virtual t_uint32 get_command_count() {
        return static_cast<t_uint32>(uieh_tabs::g_get_list_tabs().get_count());
    };
	//! Retrieves GUID of specified command.
    virtual GUID get_command(t_uint32 p_index) {
        static_api_ptr_t<hasher_md5> md5;
        pfc::string8 key_str = pfc::string8("hide_tab/Panel #") << p_index;
        return md5->process_single_guid(key_str, key_str.length());
    };
	//! Retrieves name of item, for list of commands to assign keyboard shortcuts to etc.
    virtual void get_name(t_uint32 p_index,pfc::string_base & p_out) {
        p_out = pfc::string8("Panel #") << p_index;
    };
	//! Retrieves item's description for statusbar etc.
    virtual bool get_description(t_uint32 p_index,pfc::string_base & p_out)
    {
        p_out = pfc::string8("Shows tab bar of Tab Panel #") << p_index << ".";
        return true;
    };
	//! Retrieves GUID of owning menu group.
    virtual GUID get_parent() { return mainmenu_groups_tabs::hide_tab; };
	//! Retrieves sorting priority of the command; the lower the number, the upper in the menu your commands will appear. Third party components should use sorting_priority_base and up (values below are reserved for internal use). In case of equal priority, order is undefined.
	virtual t_uint32 get_sort_priority() {return sort_priority_dontcare;}
	//! Retrieves display string and display flags to use when menu is about to be displayed. If returns false, menu item won't be displayed. You can create keyboard-shortcut-only commands by always returning false from get_display().
	virtual bool get_display(t_uint32 p_index,pfc::string_base & p_text,t_uint32 & p_flags)
    {
        if (uieh_tabs::g_get_list_tabs().get_item(p_index)->get_hide_tab()) {
            p_flags = 0;
        } else {
            p_flags = flag_checked;
        }
        get_name(p_index,p_text);
        return true;
    }
	//! Executes the command. p_callback parameter is reserved for future use and should be ignored / set to null pointer.
    virtual void execute(t_uint32 p_index,service_ptr_t<service_base> p_callback) {
        uieh_tabs::g_get_list_tabs().get_item(p_index)->toggle_hide_tab();
    };
};

class mainmenu_commands_change_height : public mainmenu_commands_hide_tab
{
public:
	virtual GUID get_command(t_uint32 p_index) {
        static_api_ptr_t<hasher_md5> md5;
        pfc::string8 key_str = pfc::string8("change_limit/Panel #") << p_index;
        return md5->process_single_guid(key_str, key_str.length());
    };
    virtual bool get_description(t_uint32 p_index,pfc::string_base & p_out)
    {
        p_out = pfc::string8("Changes limit minimum height of Tab Panel #") << p_index << ".";
        return true;
    };
    virtual GUID get_parent() { return mainmenu_groups_tabs::change_height; };
    virtual bool get_display(t_uint32 p_index,pfc::string_base & p_text,t_uint32 & p_flags)
    {
        get_name(p_index,p_text);
        return true;
    }
    virtual void execute(t_uint32 p_index,service_ptr_t<service_base> p_callback) {
        uieh_tabs::g_get_list_tabs().get_item(p_index)->change_limit_min_height();
    };
};

// {9D00418E-7627-4c5f-ADD2-DFFC1846C2DE}
const GUID mainmenu_groups_tabs::tabs = 
{ 0x9d00418e, 0x7627, 0x4c5f, { 0xad, 0xd2, 0xdf, 0xfc, 0x18, 0x46, 0xc2, 0xde } };

// {96F8E246-7782-4b27-B0F3-EDFF2B4C37BF}
const GUID mainmenu_groups_tabs::hide_tab = 
{ 0x96f8e246, 0x7782, 0x4b27, { 0xb0, 0xf3, 0xed, 0xff, 0x2b, 0x4c, 0x37, 0xbf } };

// {F4BECF67-A737-4159-8EE2-F51DBCAEB1E9}
const GUID mainmenu_groups_tabs::change_height = 
{ 0xf4becf67, 0xa737, 0x4159, { 0x8e, 0xe2, 0xf5, 0x1d, 0xbc, 0xae, 0xb1, 0xe9 } };

// {E7A7FFFD-E38E-44ea-920A-410773643F7A}
const GUID mainmenu_groups_tabs::select_tab = 
{ 0xe7a7fffd, 0xe38e, 0x44ea, { 0x92, 0xa, 0x41, 0x7, 0x73, 0x64, 0x3f, 0x7a } };

static mainmenu_group_popup_factory g_mainmenu_group_popup_tabs_factory(mainmenu_groups_tabs::tabs, mainmenu_groups::view, mainmenu_commands::sort_priority_dontcare, "Tabbed Panel Stack mod");
static mainmenu_group_popup_factory g_mainmenu_group_popup_hide_tab_factory(mainmenu_groups_tabs::hide_tab, mainmenu_groups_tabs::tabs, mainmenu_commands::sort_priority_dontcare, "Show/Hide tab");
static mainmenu_group_popup_factory g_mainmenu_group_popup_change_height_factory(mainmenu_groups_tabs::change_height, mainmenu_groups_tabs::tabs, mainmenu_commands::sort_priority_dontcare, "Limit MinHeight");
static mainmenu_group_popup_factory g_mainmenu_group_popup_select_tab_factory(mainmenu_groups_tabs::select_tab, mainmenu_groups_tabs::tabs, mainmenu_commands::sort_priority_dontcare, "Select tab");
static mainmenu_commands_factory_t<mainmenu_commands_select_tab> g_mainmenu_commands_select_tab_factory;
static mainmenu_commands_factory_t<mainmenu_commands_hide_tab> g_mainmenu_commands_hide_tab_factory;
static mainmenu_commands_factory_t<mainmenu_commands_change_height> g_mainmenu_commands_change_height_factory;